"""Pylons environment configuration"""
import os

from mako.lookup import TemplateLookup
from pylons import config

import formhelpers.lib.app_globals as app_globals
import formhelpers.lib.helpers
from formhelpers.config.routing import make_map
from formhelpers.lib.mako_forms import process_tags

def load_environment(global_conf, app_conf):
    """Configure the Pylons environment via the ``pylons.config``
    object
    """
    # Pylons paths
    root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    paths = dict(root=root,
                 controllers=os.path.join(root, 'controllers'),
                 static_files=os.path.join(root, 'public'),
                 templates=[os.path.join(root, 'templates')])

    # Initialize config with the basic options
    config.init_app(global_conf, app_conf, package='formhelpers', paths=paths)

    config['routes.map'] = make_map()
    config['pylons.app_globals'] = app_globals.Globals()
    config['pylons.h'] = formhelpers.lib.helpers

    # Create the Mako TemplateLookup, with the default auto-escaping
    config['pylons.app_globals'].mako_lookup = TemplateLookup(
        directories=paths['templates'],
        module_directory=os.path.join(app_conf['cache_dir'], 'templates'),
        input_encoding='utf-8', output_encoding='utf-8',
        imports=['from webhelpers.html import escape'],
        default_filters=['escape'],
        preprocessor=process_tags  # add "taglib" preprocessor
        )
        
    # CONFIGURATION OPTIONS HERE (note: all config options will override
    # any Pylons config options)
